package derez.libs;

import java.util.Random;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

/**
 * Note:
 * The library does not check for all possible errors in the 
 * input parameters of the methods.
 * For the cases that a check is done and the parameter does not fit –
 * the method returns the value -999.
 * In several cases the algorithm for the inverse is trying 
 * the forward method and the result may include a residual error. 
 * If accuracy is important re-check the result by implementing 
 * the forward method.
 * Some of the algorithms are taken from this site : 
 * <link>algorithms|http://statpages.org/pdfs.html</link>
 */

@ShortName("Probability")
@Author("Derez")
@Version(1)


public class Probability {
	
	/**
	 * The normal distribution, or Gaussian distribution, 
	 * is an absolutely continuous probability distribution. 
	 * The graph of the associated probability density function is  
	 * "bell"-shaped, with peak at the mean, and is known as the 
	 * Gaussian function or bell curve.
	 * The parameters μ and σ² are the mean and the variance. 
	 * The distribution with μ = 0 and σ² = 1 is called standard normal.
	 * For further information see 
	 * <link> Normal distribution|http://en.wikipedia.org/wiki/Normal_distribution</link>
	 * The library enables four types of results for the distribution, 
	 * named Modes:
	 * 1. The probability that the random variable is less than z, 
	 * defined as Left, "L".
	 * 2. The probability that the random variable is larger than z, 
	 * defined as Right, "R".
	 * 3. The probability that the random variable is between –z and z, 
	 * defined as Center, "C".
	 * 4. The probability that the random variable is less than –z 
	 * or Larger than z, defined as Tails, "L".
	 * The method returns an array of two Double variables, 
	 * the first is the Normal Probability P(z) and the second is the Density F(z).
	 */
	public double [] Normal(double Z, double Mean, double Variance, String Mode)
	{
	if ((Mode != "L") & (Mode != "C") & (Mode != "R") & (Mode != "T"))
			return new double[] { -999, -999 } ;
	return StdNormal((Z-Mean)/Math.pow(Variance, 0.5), Mode) ;
	}
	
	public  double [] StdNormal(double Z, String Mode)
	{
		/*
		 * Returns an array of two Double variables, the first is the Standard Normal Probability P(z) and the second is the Density F(z).
		 */
	if ((Mode != "L") & (Mode != "C") & (Mode != "R") & (Mode != "T"))
		return new double[] { -999, -999 } ;	
	double z2 = Z*Z ;
	double z3 = z2*Z ;
	int SignZ = 1 ;
	if (Z < 0) 
	{
		Z = Math.abs(Z) ;			
		SignZ = -1 ;
	}
	double Fn = Math.pow(Math.E,(-z2/2))/Math.pow(2*Math.PI, 0.5) ;
	double Pn = 1-0.5*Math.pow((1 + 0.0498673470 * Z 
	                              + 0.0211410061 * z2 
	                              + 0.0032776263 * z3
	                              + 0.0000380036 * z2 * z2
	                              + 0.0000488906 * z2 * z3
	                              + 0.0000053830 * z3 * z3),-16) ;
	
	if (Mode == "C") 
		Pn = 2*Pn -1 ;
	else if (Mode == "R")
		Pn = (1-Pn) ;
	else if (Mode == "L")
		{if (SignZ == -1) 
			Pn = 1-Pn ;}
	else if (Mode == "T")
		Pn = (1-Pn)*2;
	else
		if (SignZ == -1) 
			Pn = 1-Pn ;
		
	if (Pn > 1)
		Pn = 1 ;
	return new double[] { Pn, Fn } ;
	}
	
	/**
	 * Returns a Double variable Z for which the Normal Probability is P(z).
	 */
	public double  InvNormal(double Pn, double Mean, double Variance, String Mode)
	{
		if ((Mode != "L") & (Mode != "C") & (Mode != "R") & (Mode != "T"))
			return  -999 ;
		double	z = InvStdNormal(Pn, Mode) ;
		z = z * Math.pow(Variance, 0.5) + Mean ;
		return z ;
	}
	
	/**
	 * Returns a Double variable Z for which the Standard Normal Probability is P(z).
	 */
	public  double  InvStdNormal(double Pn, String Mode)
	{
	if ((Mode != "L") & (Mode != "C") & (Mode != "R") & (Mode != "T"))
		return  -999 ;			
	int SignZ = 1 ;
	int HalfP = 1 ;
	if (Pn > 0.5) 
	{
		HalfP = -1 ;
		Pn = 1-Pn ;
	}
	if (Mode == "T") 
		{if (HalfP == -1) 
			Pn = 0.5-Pn/2 ;
		else
			Pn = Pn/2 ;}
	else if (Mode == "R")
		{if (HalfP == -1)
			SignZ = -1 ;}
	else if (Mode == "L")
		{if (HalfP == 1)
			SignZ = -1 ;}
	else if (Mode == "C")
		{if (HalfP == -1)
			Pn = Pn/2 ;
		else
			Pn = 0.5-Pn/2 ;}
	else
		if (HalfP == 1)
			SignZ = -1 ;

	return SignZ * Abr(Pn) ;
	}

	/**
	 * Returns a random number based on Normal Distribution.
	 */
	public double RNGNormal1(double Mean, double Variance)
	{
	 if (Variance < 0 )
			return  -999 ;
	 double x1 = 0 ;
	 double x2 = 0 ;
	 double w = 2;
		 Random rnd = new Random() ;
     while ( w >= 1 )
         {
         	x1 = 2 * rnd.nextDouble() - 1;
            x2 = 2 * rnd.nextDouble() - 1;
            w = x1 * x1 + x2 * x2;
         }
	 w = Math.pow( -2 * Math.log(w) / w, 0.5 );
	 return x1 * w * Math.pow(Variance,0.5) + Mean;
		}
	
	/**
	  * Returns an array of two random numbers based on Normal Distribution.
	  */
	public double [] RNGNormal2(double Mean, double Variance)
	{
	 if (Variance < 0 )
			return new double [] {-999,-999} ;
	 double x1 = 0 ;
	 double x2 = 0 ;
	 double w = 2;
	 double [] res = new double [2] ;
	 Random rnd = new Random() ;
     while ( w >= 1 )
         {
         	x1 = 2 * rnd.nextDouble() - 1;
            x2 = 2 * rnd.nextDouble() - 1;
            w = x1 * x1 + x2 * x2;
         }
	 w = Math.pow( -2 * Math.log(w) / w, 0.5 );
	 res[0] = x1 * w * Math.pow(Variance, 0.5) + Mean;
	 res[1] = x2 * w * Math.pow(Variance, 0.5) + Mean;
     return res ;
		}
	
	/**
	  * Returns a random number based on Standard Normal Distribution.
	  */
	public double RNGStdNormal1()
	{
	 double x1 = 0 ;
	 double x2 = 0 ;
	 double w = 2;
		 Random rnd = new Random() ;
     while ( w >= 1 )
         {
         	x1 = 2 * rnd.nextDouble() - 1;
            x2 = 2 * rnd.nextDouble() - 1;
            w = x1 * x1 + x2 * x2;
         }
	 w = Math.pow( -2 * Math.log(w) / w , 0.5);
	 return x1 * w ;
		}
	
	/**
	  * Returns an array of two random numbers based on Standard Normal Distribution.	
	  */
	public double [] RNGStdNormal2()
	{
	 double x1 = 0 ;
	 double x2 = 0 ;
	 double w = 2;
		 Random rnd = new Random() ;
     while ( w >= 1 )
         {
         	x1 = 2 * rnd.nextDouble() - 1;
            x2 = 2 * rnd.nextDouble() - 1;
            w = x1 * x1 + x2 * x2;
         }
	 w = Math.pow( -2 * Math.log(w) / w, 0.5 );
     double y1 = x1 * w ;
     double y2 = x2 * w ;
     return new double[] {y1, y2 } ;
		}
	
	/**
	 * The Chi-Square distribution (also Chi-squared or χ²-distribution)
	 * with k degrees of freedom is the distribution of a sum of the squares
	 * of k independent standard normal random variables. 
	 * It is one of the most widely used probability distributions
	 * in inferential statistics, e.g. in hypothesis testing, 
	 * or in construction of confidence intervals.
	 * The best-known situations in which the Chi-square distribution 
	 * is used are the common Chi-square tests for goodness of fit 
	 * of an observed distribution to a theoretical one, and of the 
	 * independence of two criteria of classification of qualitative data. 
	 * Many other statistical tests also lead to a use of this distribution,
	 * like Friedman's analysis of variance by ranks.
	 * For further information see  
	 * <link>Chi square distribution|http://en.wikipedia.org/wiki/Chi-square_distribution</link>
	 * <p>
	 * This method returns an array of two Double variables, 
	 * the first is the probability that the sum of n squares of 
	 * normal distributed variables (n Degrees Of Freedom) will be less than
	 * x, and the second is the density function for x.
	 */
	public double [] ChiSq(double x, int n)
	{
	if ( x < 0)
		return new double[] { -999, -999 } ;	
	double [] res = new double [2] ;
	res[1] = ChiSqDensity( x, n) ;
	if(x>1000 | n>1000)
	{ 
		double q = Norm((Math.pow(x/n, 1/3)+ 2/(9*n)-1)
		                /Math.pow(2/(9*n), 0.5))/2;
		if (x>n)
		{
			res[0] = q ;
			return res ;
		}
		else
			{
			res[0] = 1- q ;
			return res ;
		}
	}
	double p = Math.exp(-0.5 * x);
	if((n%2)==1)
	  p = p * Math.pow(2 * x / Math.PI, 0.5);
	int k = n ;
	while(k >= 2)
    	{ 
    		p *=  x / k ;
    		k -=  2 ;
    	}
	double t = p; 
	int a = n;
	while(t > 1e-15 * p) 
    	{ 
    		a +=  2 ;
    		t *=  x / a ;
    		p +=  t ;
    	}
	res[0] = 1-p ;
	return res ;
	}
	
	/**
	 * Returns the sum of n squares of normal distributed variables 
	 * (n Degrees Of Freedom) of which probability is Pc.
	 */
	public double InvChiSq(double Pc ,int n)
	{
	double [] res = new double [2] ;
	if (Pc > 1)
		Pc = 1 ;
	double v = 0.5;
	double dv = 0.5;
	double x = 0 ;
    while(dv > 1e-10)
	    { 
	    	x = 1 / v - 1;
	    	dv = dv / 2 ;
	    	res = ChiSq(x,n);
	    	if(res[0] > Pc)
	    	  v -=  dv ;
	    	else 
	    	  v +=  dv ;
	    }
    return x ;
    }
	
	/**
	 * Student's t-distribution (or simply the t-distribution) 
	 * is a continuous probability distribution that arises in 
	 * the problem of estimating the mean of a normally distributed 
	 * population when the sample size is small. 
	 * It is the basis of the popular Student's t-tests for the 
	 * statistical significance of the difference between two sample 
	 * means, and for confidence intervals for the difference between 
	 * two population means. 
	 * For further information see 
	 * <link>t distribution|http://en.wikipedia.org/wiki/Student%27s_t-distribution</link>
	 * The library enable four types of results for the distribution, named Modes:
	 * 1. The probability that the random variable is less than z, 
	 * defined as Left, "L".
	 * 2. The probability that the random variable is larger than z, 
	 * defined as Right, "R".
	 * 3. The probability that the random variable is between –z and z, 
	 * defined as Center, "C".
	 * 4. The probability that the random variable is less than –z or 
	 * Larger than z, defined as Tails, "T".
	 * The method returns an array of two Number variables, 
	 * the first is the Student t  probability for n  Degrees Of  Freedom  
	 * for the number x with the selected Mode, 
	 * and the second is the density function for x.
	 */
	public double [] StudentT(double x,int n, String Mode)
	{
	double [] res = new double [2] ;
	double t = Math.abs(x) ;
	double w = t/Math.pow(n, 0.5);
	double th = Math.atan(w) ;
	if(n==1) 
		res[0] = 1 - 2 * th / Math.PI ;
		else
		{
			double sth = Math.sin(th);
			double cth = Math.cos(th) ;
    	if((n%2)==1)
          res[0] = 1-2*(th+sth*cth*StatCom(cth*cth,2,n-3,-1))/Math.PI ;
        else
          res[0] = 1-sth*StatCom(cth*cth,1,n-3,-1) ;
			}
				
		if (Mode == "R")
			{if(x<0)
				res[0] = 1-res[0]/2;
			else
				res[0] = res[0]/2 ;}
		else if (Mode == "L")
			{if(x>0)
				res[0] = 1-res[0]/2;
			else
				res[0] = res[0]/2 ;}
		else if (Mode == "C")
			res[0] = 1 - res[0] ;
		
		res[1] = Gammapart(n)* Math.pow(1+x*x/n , -((double)n + 1)/2) ;
		return res ;
}

	/**
	 * Returns the number for which the Probability is Ps, 
	 * according to the DOF and the selected Mode.
	 */
	public double InvStudentT(double Ps, int n, String Mode)
	{
	if (Ps > 1)
		Ps = 1 ;
	double [] res = new double [2] ;
	double v = 0.5;
	double dv = 0.5;
	double x = 0 ;
    while(dv > 1e-10)
    { 
    	x = 1 / v - 1 ;
    	dv = dv / 2 ; 
    	res = StudentT(x,n,Mode) ;
    	if(res[0] > Ps)
    		v -=  dv ;
    	else 
    		v +=  dv ;
    }
    return x;
    }
	
	/**
	 * The F-distribution is a continuous probability distribution. 
	 * It is also known as Snedecor's F distribution or the Fisher-Snedecor distribution 
	 * (after R.A. Fisher and George W. Snedecor). 
	 * The F-distribution arises frequently as the null distribution 
	 * of a test statistic, especially in likelihood-ratio tests, 
	 * perhaps most notably in the analysis of variance.
	 * It provides the probability for the ratio of two Chi-squared variables, 
	 * each with its DOF.
	 * The probability is for variables larger than the defined X 
	 * (mode "R"). 
	 * To have the left part use 1-Pf in the application.
	 * For further information see  
	 * <link>F distribution|http://en.wikipedia.org/wiki/F-distribution</link>
	 * Returns an array of two Double variables, 
	 * the first is the Fisher F probability for n1 and n2 Degrees Of Freedom  
	 * for the number f and the second is the Density function.
	 */
	public double [] FisherF(double f,int n1, int n2)
	{
	double [] res = new double [2] ;
	res[1] = FishDensity(f, n1, n2) ;
	if ((f < 0 ) | (n1< 1) | (n2<1))
		return new double[] { -999, -999 } ;	
	double x = n2 / (n1 * f + n2) ;
    if((n1%2)==0)
    {
    	res[0] =  StatCom(1-x, n2, n1+n2-4, n2-2) * Math.pow(x,(double)n2/2) ;
        return res ;
    }
    if((n2%2)==0)
    {
    	res[0] = 1 - StatCom(x, n1, n1+n2-4, n1-2) * Math.pow(1-x,(double)n1/2) ;
    	return res ;
    }
    double th = Math.atan(Math.pow(n1*f/n2, 0.5));
    double a = 2* th / Math.PI;
    double sth = Math.sin(th);
    double cth = Math.cos(th) ;
    if(n2>1)
      a +=  2 * sth * cth * StatCom(cth*cth, 2, n2-3, -1)/Math.PI ;
    if(n1==1)
    {
    	res[0] = 1-a ;
    	return res ;
    }
    double c = 4 * StatCom(sth*sth, n2+1, n1+n2-4, n2-2) * sth * Math.pow(cth,n2)/Math.PI ;
    if(n2==1)
    {
    	res[0] = 1-a+c/2 ;
    	return res ;
	}
    int k=2;
    while(k <= (n2-1)/2) 
	    {
	    c *=  k /(k-0.5);
	    k += 1;
	    }
    res[0] = 1-a+c ;
    return res ;
    }

	/**
	 * Returns the number for which the Fisher F probability is Pf.
	 */
	public double InvFisherF(double Pf, int n1, int n2)
	{
	if ((n1< 1) | (n2<1))
		return  -999 ;	
	double [] res = new double [2] ;
	double v = 0.5;
	double dv = 0.5;
	double f = 0 ;
    while(dv > 1e-10)
	    { 
	    f = 1 / v - 1 ;
	    dv = dv / 2 ; 
	    res = FisherF(f,n1,n2) ;
	    if(res[0] > Pf)
	    	v -= dv ;
	    else 
	    	v += dv ;
	    }
    return f;
    }
	
	/**
	 * The Poisson distribution is a discrete probability distribution 
	 * that expresses the probability of a number of events occurring 
	 * in a fixed period of time if these events occur with a known average rate 
	 * and independently of the time since the last event. 
	 * (The Poisson distribution can also be used for the number of events 
	 * in other specified intervals such as distance, area or volume.)
	 * For further information see 
	 *  <link>Poisson distribution|http://en.wikipedia.org/wiki/Poisson_distribution</link>
	 * The method returns an array of two Double variables, 
	 * the first is the Poisson Probability Pp(k) and 
	 * the second is the value of the Poisson Mass function for k. 
	 * Lambda is the time interval.
	 */
	public double [] Poisson(int K, double Lambda)
	{
		if (K < 0 )
			return new double[] { -999, -999 } ;
		double [] res = new double [2] ;
		res[1] = Math.pow(Lambda,K)* Math.pow(Math.E,-Lambda)/ Factorial(K) ;
		double sum = 1 ;
		for (int i=1;i<=K;i++)
			sum += 	Math.pow(Lambda,i)/Factorial(i);
		res[0] = sum * Math.pow(Math.E,-Lambda);
		return res ;
	}
	
	/**
	 * Returns an integer for which the Poisson Probability with 
	 * Lambda is Pp.
	 */
	public int InvPoisson(double Pp, double Lambda)
	{
		int k = 10;
		int dk = 8;
		int flag = 1;
		int oldflag = 1 ;
		double [] res = new double [2] ;
    	res = Poisson(k,Lambda) ;
    	while (dk >= 1)
    	{
	    	if(res[0] > Pp + 1e-6)
				flag = -1 ;
	    	else 
	    		flag = 1 ;
	    	k += flag * dk;
	    	if (k < 1 )
	    		k = 0 ;
	    	if(flag != oldflag)
	    		dk = dk/2 ;
	    	res = Poisson(k,Lambda) ;
    	}
    	if (res[0] > Pp + 1e-6 )
    		return k-1 ;
    	else
    		return k ;
	}
	
	/**
	 * Returns a random number based on Poisson distribution with Lambda.
	 */
	public int RNGPoisson(double Lambda)
	{
		double L = Math.pow(Math.E,-Lambda);
		int k = 0 ;
		double p = 1 ;
		double u ;
		Random rnd = new Random() ;
		while (p > L)
		{
         	k += 1 ;
         	u = rnd.nextDouble();
         	p *=  u ;
		}
	return k - 1 ;
	}

	/**
	 * The Exponential distributions (a.k.a. negative exponential distributions) 
	 * are a class of continuous probability distributions. 
	 * They describe the times between events in a Poisson process, 
	 * i.e. a process in which events occur continuously and independently 
	 * at a constant average rate.
	 * For further information look here: 
	 * <link>Exponential distribution|http://en.wikipedia.org/wiki/Exponential_distribution</link>
	 * The method returns an array of two Double variables, the first is 
	 * the Exponential probability for the number x with the parameter 
	 * Lambda and the second is the Density function.
	 */
	public double [] Exponential(double x, double Lambda)
	{
	if ( x < 0 | Lambda < 0)
		return new double[] { -999, -999 } ;	
	double [] res = new double [2] ;
	double a = Math.pow(Math.E,-Lambda * x) ;
	res[0] = 1- a ;
	res[1] = Lambda * a ;
	return res;
	}
	
	/**
	 * Returns the variable x for which the Exponential probability is Px.
	 */
	public double InvExponential(double Px, double Lambda)
	{
		if ( Px < 0 | Lambda < 0 | Px > 1 )
			return  -999 ;
		return -Math.log(1-Px)/Lambda ;
	}

	/**
	 * The Rayleigh distribution  is a continuous probability distribution. 
	 * As an example of how it arises, the wind speed will have a Rayleigh distribution 
	 * if the components of the two-dimensional wind velocity vector are uncorrelated 
	 * and normally distributed with equal variance. 
	 * The distribution is named after Lord Rayleigh.
	 * The distribution is a family of distributions, different by the Scale factor.
	 * For further information see 
	 *  <link>Rayleigh_distribution|http://en.wikipedia.org/wiki/Rayleigh_distribution</link>
	 * The method returns an array of two Double variables, 
	 * the first is the Rayleigh probability for the number r and 
	 * the second is the Density F(r).
	 */
	public double [] Rayleigh(double r, double Scale)
	{
		if (r < 0 | Scale < 0 )
			return new double [] {-999,-999} ;
		double [] res = new double [2] ;
		double t = Math.pow(Math.E, - r*r/(2*Scale*Scale));
		res[0] = 1- t ;
		res[1] = r * t / (Scale*Scale) ;
		return res ;
	}
	
	/**
	 * Returns the number for which the Rayleigh probability is Pr.
	 */
	public double InvRayleigh(double Pr, double Scale)
	{
		if (Scale < 0 )
			return  -999 ;
		return Math.pow(-2* Scale* Scale * Math.log(1-Pr), 0.5);
	}

	/**
	 * Provides a random number with Rayleigh Distribution characteristics.
	 */
	public double RNGRayleigh( double Scale)
	{
	 Random rnd = new Random() ;
     double u = rnd.nextDouble() ;
	 while ( u == 0 )
     	u = rnd.nextDouble();
	 return Scale * Math.pow(-2* Math.log(u), 0.5);
		}
	
	/**
	 * The Erlang distribution is a continuous probability distribution 
	 * with wide applicability primarily due to its relation to the exponential and Gamma distributions. 
	 * The Erlang distribution was developed by A. K. Erlang to examine 
	 * the number of telephone calls which might be made at the same time 
	 * to the operators of the switching stations. 
	 * This work on telephone traffic engineering has been expanded to consider 
	 * waiting times in queueing systems in general. 
	 * The distribution is now used in the fields of stochastic processes 
	 * and of biomathematics.
	 * The distribution is a continuous distribution, which has a positive value 
	 * for all real numbers greater than zero, and is given by two parameters: 
	 * the shape k, which is a non-negative integer, 
	 * and the rate λ, which is a non-negative real number. 
	 * The distribution is sometimes defined using the inverse of the rate parameter, 
	 * the scale μ.
	 * For further information see 
	 * <link>Erlang distribution|http://en.wikipedia.org/wiki/Erlang distribution</link>
     * The method returns an array of two Double variables, 
	 * the first is the Erlang probability for the number x 
	 * and the second is the Density F(x).
	 */
	public double [] Erlang(double x, int k, double Lambda)
	{
		if ( x < 0 | Lambda < 0 | k < 1)
			return new double[] { -999, -999 } ;
		double [] res = new double [2] ;
		res[1] = Math.pow(Lambda,k)*Math.pow(x,k-1)*Math.pow(Math.E,-Lambda*x) 
			/ Factorial(k-1) ;
		double sum = 0;
		for (int n = 0;n< k ; n++)
			sum += Math.pow(Lambda*x,n)/Factorial(n) ;
		res[0] = 1- sum * Math.pow(Math.E, -Lambda*x);
		return res ;
	}
	
	/**
	 * Returns the number for which the Erlang probability is Px.
	 */
	public double InvErlang(double Px,int k, double Lambda)
	{
	if ((Px< 0) | (Px>1) | (Lambda < 0) | (k<1) )
		return  -999 ;	
	double [] res = new double [2] ;
	double v = 0.5;
	double dv = 0.5;
	double x = 0 ;
    while(dv > 1e-10)
	    { 
	    x = 1 / v - 1 ;
	    dv = dv / 2 ; 
	    res = Erlang(x,k,Lambda) ;
	    if(res[0] < Px)
	    	v -= dv ;
	    else 
	    	v += dv ;
	    }
    return x;
	}

	/**
	 * The Binomial distribution is the discrete probability distribution 
	 * of the number of successes in a sequence of n independent yes/no experiments, 
	 * each of which yields success with probability  p. 
	 * Such a success/failure experiment is also called a Bernoulli experiment 
	 * or Bernoulli trial. 
	 * The binomial distribution is the basis for the popular binomial test 
	 * of statistical significance.
	 * It is frequently used to model number of successes in a sample 
	 * of size n from a population of size N. 
	 * Since the samples are not independent (this is sampling without replacement), 
	 * the resulting distribution is a hypergeometric distribution, 
	 * not a binomial one. However, for N much larger than n, 
	 * the binomial distribution is a good approximation, and widely used.
	 * For further information see 
	 * <link>Binomial_distribution|http://en.wikipedia.org/wiki/Binomial_distribution</link>
	 * The method returns an array of two Double variables, the first 
	 * is the Binomial probability for k out of n trials 
	 * with probability of p for each trial, and the second – 
	 * the Binomial Mass function for the number k.
	 */
	public double [] Binomial(int k, int n, double p)
	{
		if(p>=1 | p < 0 | n< 0 | k < 0)
			return new double [] {-999,-999} ;
		double [] res = new double [2] ;
		if(n<60 )
		{
	    	res[1] = NChooseK(n,k)* Math.pow(p,k) * Math.pow(1-p,n-k);
			double sum = 0 ;
			
			for (int i=0;i<k+1;i++)
				sum += NChooseK(n,i) *Math.pow(p,i) * Math.pow(1-p,n-i) ;
			res[0] = sum ;
		}
		else
			res = Normal(k, n*p,(1-p)*n*p, "L") ;
		return res ;	
	}
	
	/**
	 * Returns the number for which the Binomial Probability, 
	 * with n trials and trial’s probability p, is Pb.
	 */
	public int InvBinomial(double Pb, int n, double p)
	{
		if(p>=1 | p < 0 | n< 0 | Pb < 0 | Pb > 1 )
			return -999 ;
		double [] res = new double [2] ;
		if (Pb > 0.9999)
		{
			int i = 1 ;
			res = Binomial( n-i, n, p);
			while(res[0] > 0.9999)
			{
			i++ ;
			res = Binomial( n-i, n, p) ;
			}
			return n-i+1 ;
		}
		else
		{
			int k = 10;
			int dk = 8;
			int flag = 1;
			int oldflag = 1 ;
	    	res = Binomial(k,n,p) ;
	    	while (dk >= 1)
	    	{
		    	if(res[0] > Pb + 5e-5)
					flag = -1 ;
		    	else 
		    		flag = 1 ;
		    	k += flag * dk;
		    	if (k < 1 )
		    		k = 0 ;
		    	if (k > n)
		    		k = n-1;
		    	if(flag != oldflag)
		    		dk = dk/2 ;
		    	res = Binomial(k,n,p) ;
	    	}
	    	if (res[0] > Pb + 5e-5 )
	    		return k-1 ;
	    	else
	    		return k ;
		}
	}
	
	/**
	 * The Geometric distribution is either of two discrete probability 
	 * distributions:
	 * 1. The probability distribution of the number X of Bernoulli trials 
	 * needed to get one success, supported on the set { 1, 2, 3, ...}
	 * 2. The probability distribution of the number Y = X − 1 of 
	 * failures before the first success, supported on the set { 0, 1, 2, 3, ... }
	 * Which of these one calls "the" geometric distribution is a matter of convention 
	 * and convenience.
	 * These two different geometric distributions should not be confused with each other.
	 * Often, the name shifted geometric distribution is adopted for the first one 
	 * (distribution of the number X); however, to avoid ambiguity, 
	 * it is considered wise to indicate which is intended, by mentioning the range explicitly.
	 * The library use the parameter Shifted as explained above, 
	 * with values 0 for not shifted (var. Y) and 1 for shifted, var X .
	 * For further information see 
	 * <link>Geometric_distribution|http://en.wikipedia.org/wiki/Geometric_distribution</link>
	 * The method returns an array of two Double variables, the first 
	 * is the Geometric probability for k trials with probability of p 
	 * for each trial, and the second the Geometric Mass function for the number k.
	 */
	public double [] Geometric(int k, double p, int Shifted)
	{
		if(p>=1 | p < 0 | k < 0 | Shifted < 0 | Shifted > 1)
			return new double [] {-999,-999} ;
		double [] res = new double [2] ;
		res[0] = 1 - Math.pow(1-p,k+1 -Shifted) ;
		res[1] = p * Math.pow(1-p,k-Shifted) ;
		return res ;
	}
	
	/**
	 * Returns the number for which the Geometric probability, 
	 * with trial’s probability p, is Pg.
	 */
	public int InvGeometric(double Pg, double p, int Shifted)
	{
		if(p>=1 | p < 0 | Pg < 0 | Pg > 1 | Shifted < 0 | Shifted > 1)
			return -999 ;
		double t = Math.log(1-Pg)/Math.log(1-p)-1 + Shifted ;
		return (int)(t) ;
	}

	/**
	 * This methods is a mathematical function which calculates and 
	 * returns the number of different possibilities to choose 
	 * a group of k out of a group of n. 
	 * The formula is n! / (k! * (n-k)! ).
	 * See here <link>N_choose_k |http://en.wikipedia.org/wiki/N_choose_k</link>
	 */
	public double NChooseK(int n,int k)
	{
		return Factorial(n)/(Factorial(k)* Factorial(n-k)) ;
	}
	
	/**
	 * May be used for integers or integers + 0.5 .
	 * See here <link>Beta_function|http://en.wikipedia.org/wiki/Beta_function</link>
	 */
	public double Beta(double m, double n)  // for m,n integer or integer+0.5
	{
		return Gamma(m)*Gamma(n) / Gamma(m+n) ;
	}
	
	/**
	 * May be used for integers or integers + 0.5 .
	 * See here <link>Gamma_function|http://en.wikipedia.org/wiki/Gamma_function</link>
	 */
	public double Gamma(double k)  // for k integer or integer+0.5
	{
		if (k == Math.floor(k) )
			return Factorial((int)k - 1) ;
		else
		{
			int m = (int)(k - 0.5) ;
			return DblFactorial( m)
				* Math.pow(Math.PI, 0.5) / Math.pow(2,k-0.5);
		}
	}
	
	public double Factorial(int k)
	{
		if(k<0)
			return -999 ;
		double mul = 1 ;
		if(k == 0)
			return 1 ;
		else
		{
			for (int i = 1; i <= k; i++)
			  mul *= i ;
			return mul ;
		}
	}

	/**
	 * See here <link>Double_factorial|http://en.wikipedia.org/wiki/Double_factorial#Double_factorial</link>
	 */
	public double DblFactorial(int n)
	{
		return Factorial(2*n)/(Factorial(n) * Math.pow(2,n));
	}
		
	private double FishDensity(double x, int n1, int n2)
	{
		double nom = Math.pow(Math.pow(n1*x,n1)
		                       *Math.pow(n2,n2)
		                       /Math.pow(n1*x + n2,n1+n2), 0.5);
		double denom = x * Beta((double)n1/2,(double)n2/2) ;
		return nom/denom ;
	}
	
	private double Gammapart(int n)
	{
		double nom ;
		double denom ;
		int i = 1 ;
		if(n%2 == 0)
		{
			nom = 1 ;
			while (n-i > 1)
			{
				nom *= n-i ;
				i +=2 ;
			}
			denom = 2* Math.pow(n, 0.5) ;
			i = 2 ;
			while(n-i > 0)
			{
				denom *= n-i ;
				i += 2 ;
			}
		}
		else
		{
			nom = 1 ;
			while (n-i > 0)
			{
				nom *= n-i ;
				i +=2 ;
			}
			denom = Math.PI * Math.pow(n, 0.5) ;
			i = 2 ;
			while(n-i > 1)
			{
				denom *= n-i ;
				i += 2 ;
			}
		}
		return nom / denom ;
	}
	
	private double StatCom(double q,int i,int j,int b) 
	{
    double zz = 1 ;
    double z = zz ;
    int k = i ;
    while(k <= j)
	    { 
	    	zz *=  q * k / (k-b) ;
	    	z  +=  zz ;
	    	k  +=  2 ;
	    }
    return z ;
    }
	
	private double Norm(double z) 
	{
	double [] res = new double [2] ;
	double q = z*z ;
    if(Math.abs(z)>7)
       	return (1-1/q+3/(q*q))*Math.exp(-q/2)/(Math.abs(z)*Math.pow(Math.PI/2, 0.5));
    else
	    {
	    	res = ChiSq(q,1) ;
	    	return res[0] ;
	    }
    }
	
	private double Abr(double Pz)
	{
	double t = Math.log(1/(Pz*Pz)) ;
	double tr = Math.pow(t,0.5) ;
	double	Nom = 2.515517 + 0.802853 * tr + 0.010328 * t;
	double 	Denom = 1 + 1.432788 * tr + 0.189269 * t + 0.001308 *t*tr ;
	return  tr - Nom / Denom ;
	}
	
	private double ChiSqDensity(double x, int n)
	{
	double kh = (double)n/2 ;
	return Math.pow(x,kh-1)* Math.pow(Math.E,-x/2)
		/(Math.pow(2,kh) * Gamma(kh)) ;
	}
}
